Calculates accuracy and related metrics.
Arguments
- tabble
A frequency table created with
table
Details
Calculates accuracy, lower and upper bounds, the guessing rate and
p-value of the accuracy vs. the guessing rate. This function is called by
confusion_matrix, but if this is all you want, you can simply supply
the table to this function.
Author
Michael Clark (see m-clark/confusion_matrix).